/****************************************************************/
/*                                                              */
/*  file: rcuscope.c  new                                       */
/*                                                              */
/* This program allows to access the resources of a RCU         */
/* card in a user friendly way and includes some test routines  */
/*                                                              */
/*  Author: Markus Joos, CERN-EP                                */
/*                                                              */
/*  16. Mar. 04  MAJO  created                                  */
/*                                                              */
/****************C 2004 - A nickel program worth a dime**********/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <signal.h>
#include <iostream>
#include <fstream>
#include "get_input.h"
#include "rcu_lib.h"
#include "u2f.h"

using namespace std;

/*************/
/* Externals */
/*************/
extern u_int packageId;
extern u_int traceLevel;

/*************/
/* Prototypes*/
/*************/
int mainhelp(void);
int dumpdesc(void);
int dumpmem(void);
int u2fmenu(void);
int rcumenu(void);
int u2ffuncmenu(void);
int funcmenu(void);
int setdebug(void);


/****************/
int setdebug(void)
/****************/
{
  printf("Package identifiers:\n");
  printf("RCU library: %d\n", P_ID_RCU);
  printf("U2F library: %d\n", P_ID_U2F);
  printf("\n");
  printf("Enter the debug package: ");
  packageId = getdecd(packageId);
  printf("Enter the debug level: ");
  traceLevel = getdecd(traceLevel);
  return(0);
}


/****************/
int mainhelp(void)
/****************/
{
  printf("Call Markus Joos, 72364, 160663 if you need help\n");
  return(0);
}


/****************/
int dumpdesc(void)
/****************/
{
  return(0);
}


/***************/
int dumpmem(void)
/***************/
{
  int ret, loop;
  rcu_bulk_in_t in;
  
  ret = RCU_Open();
  if (ret)
  {
    rcc_error_print(stdout, ret);
    return(-1);
  }

  ret = RCU_Get(&in);
  if (ret)
  {
    rcc_error_print(stdout, ret);
    return(-1);
  }
  
  printf("%d bytes received\n", in.nbytes);
  for (loop = 0; loop < in.nbytes; loop++)
  {
    printf("Byte %d = %x\n", loop, in.data[loop]);
  }
  
  ret = RCU_Close();
  if (ret)
  {
    rcc_error_print(stdout, ret);
    return(-1);
  }  

  return(0);
}


/******************************/
int main(int argc, char *argv[])
/******************************/
{  
  static int ret, fun = 1;
  
  printf("\n\n\nThis is RUCSCOPE\n");
  while(fun != 0)
  {
    printf("\n");
    printf("Select an option:\n");
    printf("  1 Print help\n");
    printf("  2 Goofie\n");
    printf("  3 Set debugging parameters\n");
    printf("  0 Quit\n");
    printf("Your choice ");
    fun = getdecd(fun);
    if (fun == 1) mainhelp();
    if (fun == 2) u2fmenu();
    if (fun == 3) setdebug();
  }

  exit(0);
}


/***************/
int u2fmenu(void)
/***************/
{  
  static int ret, fun;
 
  fun = 1; 
  printf("\n\nThis is the sub-menu for the functions of the USB2FEC library\n");
  while(fun != 0)
  {
    printf("\n");
    printf("Select an option:\n");
    printf("  1 Execute funtions\n");
    printf("  0 Quit\n");
    printf("Your choice ");
    fun = getdecd(fun);
    if (fun == 1) u2ffuncmenu();
  }

  return(0);
}


/*******************/
int u2ffuncmenu(void)
/*******************/
{  
  static u_int ret, fun, rcode = 0, rdata = 0, fdata[2048] = {0}, fsize = 0, ccode = 0, cdata = 0;
  static u_int offset = 0, rsize = 1, memdata[2048] = {0}, dsource = 0;
  static char fname[200] = {0};
  static errst_t errst_s;
  static trcfg_t trcfg_s;
  static trcnt_t trcnt_s;
  u_int loop, osize, cnt;
 
  fun = 1; 
  while(fun != 0)
  {
    printf("\n");
    printf("Select an option:\n");
    printf("   1 Goofie_Open	2 Goofie_Close\n");
    printf("   3 PMEM_Read		4 PMEM_Write  \n");
    printf("   5 DMEM_Read          	               \n");
    printf("   6 Reg_Read		7 Reg_Write\n");
    printf("   8 EVL_Read			       \n");
    printf("   9 Exec_Command      		       \n");
    printf("  10 File_Read		11 File_Write  \n");
    printf("  ==========================================\n");
    printf("   0 Quit\n");
    printf("Your choice ");
    fun = getdecd(fun);
    if (fun == 1) 
    {
      ret = U2F_Open();
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      }
    }
    
    if (fun == 2) 
    {
      ret = U2F_Close();
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      }
    }    
 
    if (fun == 3) 
    {
      
      /* Modified by Antoine the 10/11/04 */
      
      int ch,add,offset;
      
      ch    = 1;
      add   = 0;
      rsize = 1;
      
      
      printf("Enter the channel number ( 1, 2 or 3 ) you want to read:");
      ch = getdecd(ch);
      printf("Enter the address' word:");
      add = getdecd(add);
      
      if (ch == 1)
      	{
		ch = 0;
	}
      
      else if (ch == 2)
      	{
		ch = 1024;
	}
	
      else if (ch == 3)
      	{
		ch = 2048;
	}
	else	
      	printf("ERROR: rcode out of range \n");
	
      offset = ch + add;
	
      ret = U2F_PMEM_Read(rsize, &osize, offset, memdata);
      if (ret)
      {
        printf("%d words received\n", osize);
        rcc_error_print(stdout, ret);
        return(-1);
      }
      printf("%d words received. Dumping data\n", osize);
      for (loop = 0; loop < osize; loop++)
        printf("Word %d = 0x%04x\n", loop, memdata[loop]);
    }    

    if (fun == 4)
    {
      /* Modified by Antoine the 10/11/04 */
      
      int ch,add,offset;
      
      ch    = 1;
      add   = 0;
      rsize = 1;
      
      
      printf("Enter the channel number ( 1, 2 or 3 ) you want to write: ");
      ch = getdecd(ch);
      
      printf("Enter the address' word:");
      add = getdecd(add);
      
      if (ch == 1)
      	{
		ch = 0;
	}
      
      else if (ch = 2)
      	{
		ch == 1024;
	}
	
      else if (ch = 3)
      	{
		ch == 2048;
	}
      else	
      	printf("ERROR: rcode out of range \n");
	
      offset = ch + add;
      
      printf("Select the data source (0 = Enter by hand  1 = From last U2F_File_Read): ");
      dsource = getdecd(dsource);
 
      if (!dsource)
      {
        for (loop = 0; loop < rsize; loop++)
	{
	  printf("Enter data word %d: 0x", loop);
	  memdata[loop] = gethexd(memdata[loop]);
	}
      }
      else
      {
        for (loop = 0; loop < rsize; loop++)
	  memdata[loop] = fdata[loop];
      }

      ret = U2F_PMEM_Write(rsize, offset, memdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      }
    }      
 
    if (fun == 5)
    {    
      printf("Enter the channel number ( 1, 2 or 3 ) you want to read: ");
      offset = getdecd(offset);
      
      if ( offset == 1 )
      	{
		offset = 0;
	}
	
      else if ( offset == 2 )
      	{
		offset = 1024;
	}
	
      else if ( offset == 3 )
      	{
		offset = 2048;
	}
      else	
      	printf("ERROR: rcode out of range \n");
	
      printf("How many words do you want to read: ");
      rsize = getdecd(rsize);
      printf("rsize = %d\n", rsize);
      ret = U2F_DMEM_Read(rsize, &osize, offset, memdata);
      if (ret)
      {
        printf("%d words received\n", osize);
        rcc_error_print(stdout, ret);
        return(-1);
      }
      //file 
      fstream file;
      int yesFile = 0;
      printf("Enter '1' if you want to have a file with data from mem : ");
      if(getdecd(yesFile) == 1) {
	printf("Enter the name of file : ");
	char name[20];
	fgets(name,20,stdin); 
	if (name[strlen(name)-1] == '\n') name[strlen(name)-1] = '\0' ;
	file.open(name, ios::out);
	yesFile = 1;
      }
      
      printf("%d words received. Dumping data\n", osize);
      for (loop = 0; loop < osize; loop++) {
        printf("Word %d = 0x%08x\n", loop, memdata[loop]);
	if(yesFile) {
	  file << memdata[loop] << endl; 
	}
      }
      if(yesFile)	    
        file.close();
    }     

    if (fun == 6)
    {    
      printf("Register | code\n");
      printf("=========|=====\n");
      printf("BSLCT    | %d\n", O_BSLCT);
      printf("TMASK    | %d\n", O_TMASK);
      printf("ACTCH    | %d\n", O_ACTCH);
      printf("ZSCTL    | %d\n", O_ZSCTL);
      printf("ZSTHD    | %d\n", O_ZSTHD);
      printf("ZSPED    | %d\n", O_ZSPED);
      printf("TATHD    | %d\n", O_TATHD);
      printf("TBTHD    | %d\n", O_TBTHD);
      printf("NSEVT    | %d\n", O_NSEVT);
      printf("TRGEV    | %d\n", O_TRGEV);
      printf("PRTRG    | %d\n", O_PRTRG);
      printf("Enter the code of the register you want to read: ");
      rcode = getdecd(rcode);      
    
      ret = U2F_Reg_Read(rcode, &rdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      } 
      
      printf("The register contains 0x%08x\n", rdata);    
    }     
 
    if (fun == 7)
    {    
      printf("Register | code\n");
      printf("=========|=====\n");
      printf("BSLCT    | %d\n", O_BSLCT);
      printf("TMASK    | %d\n", O_TMASK);
      printf("ACTCH    | %d\n", O_ACTCH);
      printf("ZSCTL    | %d\n", O_ZSCTL);
      printf("ZSTHD    | %d\n", O_ZSTHD);
      printf("ZSPED    | %d\n", O_ZSPED);
      printf("TATHD    | %d\n", O_TATHD);
      printf("TBTHD    | %d\n", O_TBTHD);
      printf("NSEVT    | %d\n", O_NSEVT);
      printf("PRTRG    | %d\n", O_PRTRG);
      printf("Enter the code of the register you want to modify: ");
      rcode = getdecd(rcode);      
      printf("Enter data that you want to write: 0x");
      rdata = gethexd(rdata);
  
      ret = U2F_Reg_Write(rcode, rdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      }
    }     
 
     if (fun == 8)
    {    
      /* modified by Antoine th 10/11/04 */
      
      printf("Register | code\n");
      printf("=========|=====\n");
      printf("CHANNEL_A|  %d\n", 1 ); /* O_CHA */
      printf("CHANNEL_B|  %d\n", 2 ); /* O_CHB */
      printf("CHANNEL_C|  %d\n", 3 ); /* O_CHC */
      printf("Enter the address of the channel you want to read: ");
      rcode = getdecd(rcode);      
    
     if ( rcode == 1 )
     	{
		rcode = 0;
	}
    
     else if ( rcode == 2 )
     	{
		rcode = 1024;
	}
	
     else if ( rcode == 3 )
     	{
		rcode = 2048;
	}
     else	
      	printf("ERROR: rcode out of range \n");
	
      ret = U2F_Evl_Read(rcode, &rdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      } 
      
      printf("The register contains 0x%08x\n", rdata);    
    }     


    if (fun == 9)
    {    
      /* modified by Antoine the 10/11/04 */
            
      printf("Command   | code\n");
      printf("==========|=====\n");
      printf("SWTRG     |  %d\n", 1 ); /* C_SWTRG */
      printf("SWRST     |  %d\n", 2 ); /* C_SWRST */
      printf("TWRST     |  %d\n", 3 ); /* C_TWRST */
      printf("Enter the code of the command command:");
      
      rcode = getdecd(rcode);
      
      if (rcode == 1) 
      	{ 
		ccode = C_SWTRG;
	}
      else if (rcode == 2) 
      	{
		ccode = C_SWRST;
	}
	
      else if (rcode == 3) 
      	{
		ccode = C_TWRST;
	}
      else	
      	printf("ERROR: rcode out of range \n");
      
      /* ccode = gethexd(ccode); */
      
      ret = U2F_Exec_Command(ccode, cdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      } 
    }

    if (fun == 10)
    {    
      printf("Enter the path and name of the file to read: ");
      getstrd(fname, fname);
      printf("Enter the number of words to read: ");
      fsize = getdecd(fsize);

      ret = U2F_File_Read(fname, fsize, fdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      }
      printf("The file has been read into the internal array\n"); 
    }     
 
    if (fun == 11)
    {    
      printf("Enter the path and name of the file to write: ");
      getstrd(fname, fname);
      printf("Enter the number of words to write: ");
      fsize = getdecd(fsize);

      ret = U2F_File_Write(fname, fsize, fdata);
      if (ret)
      {
        rcc_error_print(stdout, ret);
        return(-1);
      }
      printf("The file has been written with data from the internal array\n");       
    }         

  }
  return(0);
}





